# Copyright (c) 2021 T.Furukawa
# This software is released under the MIT License, see LICENSE.

class Entry:

    def __init__(self, title, mediawiki):
        self.title = title
        self.mediawiki = mediawiki
