# Copyright (c) 2021 T.Furukawa
# This software is released under the MIT License, see LICENSE.

class Redirection:

    def __init__(self, source, target):
        self.source = source
        self.target = target
